using System;
using System.Collections;
using System.IO;
using System.Text;
using System.Linq;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Windows.Forms;

using Do.Universe;
using Do.Platform;

using Gnome.Keyring;

namespace Parola
{
	public class CreateAction : Act
	{
		public override string Name {
			get { return "Create Account"; }
		}

		public override string Description {
			get { return "Create account."; }
		}

		public override string Icon {
			get { return "seahorse";}
		}

		public override IEnumerable<Type> SupportedItemTypes {
			get {
				yield return typeof(ITextItem);
			}
		}

		public override IEnumerable<Item> Perform (IEnumerable<Item> items, IEnumerable<Item> modifierItems)
		{
			System.Uri url = new System.Uri("http://" + (items.First() as ITextItem).Text);
			String account_name = url.Host;
			String[] userInfo = url.UserInfo.Split(':');
			String username = "";
			String password = "";

			if (account_name == "" || url.UserInfo == "") return null;

			if (userInfo.Count() == 1)
			{
				username = userInfo[0];
				password = RandomPassword.Generate(6, 10);

			}else if (userInfo.Count() == 2)
			{
				username = userInfo[0];
				password = userInfo[1];
			}

			Hashtable table = new Hashtable();
			table["username"] = username;
			table["account_name"] = account_name;
			table["password"] = password;

			Ring.CreateItem(Ring.GetDefaultKeyring(), ItemType.GenericSecret, account_name, table, password, false);

			Parola parola = new Parola();
			parola.setClipboardData(password);

			return null;
		}
	}

	public class DeleteAction : Act
	{
		public override string Name {
			get { return "Delete Account"; }
		}

		public override string Description {
			get { return "Delete selected account."; }
		}

		public override string Icon {
			get { return "seahorse";}
		}

		public override IEnumerable<Type> SupportedItemTypes {
			get {
				yield return typeof(ParolaItem);
			}
		}

		public override IEnumerable<Item> Perform (IEnumerable<Item> items, IEnumerable<Item> modifierItems)
		{
			ParolaItem item = (items.First() as ParolaItem);

			Ring.DeleteItem(Ring.GetDefaultKeyring(), item.AccountId);

			Gtk.Application.Invoke ((o, e) => Services.Notifications.Notify(Name, "Account deleted."));

			return null;
		}
	}

	public class CopyPasswordAction : Act
	{
		public override string Name {
			get { return "Copy Password"; }
		}

		public override string Description {
			get { return "Copy password of selected account."; }
		}

		public override string Icon {
			get { return "seahorse";}
		}

		public override IEnumerable<Type> SupportedItemTypes {
			get {
				yield return typeof(ParolaItem);
			}
		}

		public override IEnumerable<Item> Perform (IEnumerable<Item> items, IEnumerable<Item> modifierItems)
		{
			ParolaItem item = items.First() as ParolaItem;

			Parola parola = new Parola();
			parola.setClipboardData(item.Password);
			
			Gtk.Application.Invoke ((o, e) => Services.Notifications.Notify(Name, "Copied password."));

			return null;
		}
	}

	public class FindAction : Act
	{
		public override string Name {
			get { return "Find Account"; }
		}

		public override string Description {
			get { return "Find account."; }
		}

		public override string Icon {
			get { return "seahorse";}
		}

		public override IEnumerable<Type> SupportedItemTypes {
			get {
				yield return typeof(ITextItem);
			}
		}

		public override IEnumerable<Item> Perform (IEnumerable<Item> items, IEnumerable<Item> modifierItems)
		{
			String query = (items.First() as ITextItem).Text;

			Parola parola = new Parola();
			IEnumerable<ParolaItem> parola_items = parola.Find(query);

			if (!parola_items.Any ()) {
				Gtk.Application.Invoke ((o, e) => Services.Notifications.Notify(Name, "Your search did not match any result."));
			}

			foreach (ParolaItem item in parola_items) {
				yield return item;
			}
		}
	}
}


